@@ -154,7 +154,8 @@ void KSMServer::shutdown( KWorkSpace::ShutdownConfirm confirm,
     if ( !logoutConfirmed ) {
         KApplication::kApplication()->updateUserTimestamp();
         KSMShutdownFeedback::start(); // make the screen gray
-        logoutConfirmed = KSMShutdownDlg::confirmShutdown( maysd, choose, sdtype, bopt );
+        QString theme = cg.readEntry( "theme", "default" );
+        logoutConfirmed = KSMShutdownDlg::confirmShutdown( maysd, choose, sdtype, bopt, theme);
         // ###### We can't make the screen remain gray while talking to the apps,
         // because this prevents interaction ("do you want to save", etc.)
         // TODO: turn the feedback widget into a list of apps to be closed,
@@ -206,16 +207,14 @@ void KSMServer::shutdown( KWorkSpace::ShutdownConfirm confirm,
             // before others a chance to change anything.
             // KWin will check if the session manager is ksmserver,
             // and if yes it will save in phase 1 instead of phase 2.
-            if( isWM( c ) )
+            if( isWM( c )) {
                 ++wmPhase1WaitingCount;
-        }
-        if (wmPhase1WaitingCount > 0) {
-            foreach( KSMClient* c, clients ) {
-                if( isWM( c ) )
-                    SmsSaveYourself( c->connection(), saveType,
-                                true, SmInteractStyleAny, false );
+                SmsSaveYourself( c->connection(), saveType,
+                            true, SmInteractStyleAny, false );
             }
-        } else { // no WM, simply start them all
+
+        }
+        if( wmPhase1WaitingCount == 0 ) { // no WM, simply start them all
             foreach( KSMClient* c, clients )
                 SmsSaveYourself( c->connection(), saveType,
                             true, SmInteractStyleAny, false );
@@ -250,15 +249,12 @@ void KSMServer::saveCurrentSession()
 #endif
     foreach( KSMClient* c, clients ) {
         c->resetState();
-        if( isWM( c ) )
+        if( isWM( c )) {
             ++wmPhase1WaitingCount;
-    }
-    if (wmPhase1WaitingCount > 0) {
-        foreach( KSMClient* c, clients ) {
-            if( isWM( c ) )
-                SmsSaveYourself( c->connection(), saveType, false, SmInteractStyleNone, false );
+            SmsSaveYourself( c->connection(), saveType, false, SmInteractStyleNone, false );
         }
-    } else {
+    }
+    if( wmPhase1WaitingCount == 0 ) {
         foreach( KSMClient* c, clients )
             SmsSaveYourself( c->connection(), saveType, false, SmInteractStyleNone, false );
     }
