@@ -1630,9 +1630,9 @@ void Client::setOnActivities(QStringList newActivitiesList)
     newActivitiesList = joinedActivitiesList.split(',', QString::SkipEmptyParts);
 
     QStringList allActivities = workspace()->activityList();
-    if (newActivitiesList.size() == allActivities.size() || newActivitiesList.isEmpty() ||
+    if ( newActivitiesList.isEmpty() ||
+        (newActivitiesList.count() > 1 && newActivitiesList.count() == allActivities.count()) ||
         (newActivitiesList.count() == 1 && newActivitiesList.at(0) == "ALL")) {
-        setOnAllActivities(true);
         activityList.clear();
         XChangeProperty(display(), window(), atoms->activities, XA_STRING, 8,
                         PropModeReplace, (const unsigned char *)"ALL", 3);
