@@ -321,6 +321,13 @@ public:
      **/
     void popShader();
 
+    /**
+     * Resets all shaders to the default uniform values.
+     * Only built in shaders are changed.
+     * @since 4.8
+     **/
+    void resetAllShaders();
+
     /**
      * Creates a GLShader with a built-in vertex shader and a custom fragment shader.
      * @param vertex The generic vertex shader
@@ -503,9 +510,10 @@ public:
      */
     void render(GLenum primitiveMode);
     /**
-     * Same as above restricting painting to @a region.
+     * Same as above restricting painting to @a region if @a hardwareClipping is true.
+     * It's within the caller's responsibility to enable GL_SCISSOR_TEST.
      */
-    void render(const QRegion& region, GLenum primitiveMode);
+    void render(const QRegion& region, GLenum primitiveMode, bool hardwareClipping = false);
     /**
      * Sets the color the geometry will be rendered with.
      * For legacy rendering glColor is used before rendering the geometry.
