@@ -34,6 +34,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // Qt
 #include <QAction>
 #include <QX11Info>
+#include <QtDBus/QDBusConnection>
 // KDE
 #include <KActionCollection>
 #include <KConfig>
@@ -120,19 +121,109 @@ QWeakPointer<TabBoxClient> TabBoxHandlerImpl::activeClient() const
         return QWeakPointer<TabBoxClient>();
 }
 
-QWeakPointer<TabBoxClient> TabBoxHandlerImpl::clientToAddToList(TabBoxClient* client, int desktop, bool allDesktops) const
+bool TabBoxHandlerImpl::checkDesktop(TabBoxClient* client, int desktop) const
 {
+    Client* current = (static_cast< TabBoxClientImpl* >(client))->client();
+
+    switch (config().clientDesktopMode()) {
+    case TabBoxConfig::AllDesktopsClients:
+        return true;
+    case TabBoxConfig::ExcludeCurrentDesktopClients:
+        return !current->isOnDesktop(desktop);
+    default:       // TabBoxConfig::OnlyCurrentDesktopClients
+        return current->isOnDesktop(desktop);
+    }
+}
+
+bool TabBoxHandlerImpl::checkActivity(TabBoxClient* client) const
+{
+    Client* current = (static_cast< TabBoxClientImpl* >(client))->client();
+
+    switch (config().clientActivitiesMode()) {
+    case TabBoxConfig::AllActivitiesClients:
+        return true;
+    case TabBoxConfig::ExcludeCurrentActivityClients:
+        return !current->isOnCurrentActivity();
+    default:       // TabBoxConfig::OnlyCurrentActivityClients
+        return current->isOnCurrentActivity();
+    }
+}
+
+bool TabBoxHandlerImpl::checkApplications(TabBoxClient* client) const
+{
+    Client* current = (static_cast< TabBoxClientImpl* >(client))->client();
+    TabBoxClientImpl* c;
+    QListIterator< QWeakPointer<TabBoxClient> > i(clientList());
+
+    switch (config().clientApplicationsMode()) {
+    case TabBoxConfig::OneWindowPerApplication:
+        // check if the list already contains an entry of this application
+        while (i.hasNext()) {
+            QSharedPointer<TabBoxClient> client = i.next().toStrongRef();
+            if (!client) {
+                continue;
+            }
+            if ((c = dynamic_cast< TabBoxClientImpl* >(client.data()))) {
+                if (c->client()->resourceClass() == current->resourceClass()) {
+                    return false;
+                }
+            }
+	}
+        return true;
+    case TabBoxConfig::AllWindowsCurrentApplication: {
+        QSharedPointer<TabBoxClient> pointer = tabBox->activeClient().toStrongRef();
+        if (!pointer) {
+            return false;
+        }
+        if ((c = dynamic_cast< TabBoxClientImpl* >(pointer.data()))) {
+            if (c->client()->resourceClass() == current->resourceClass()) {
+                return true;
+            }
+        }
+        return false;
+    }
+    default:       // TabBoxConfig::AllWindowsAllApplications
+      return true;
+    }
+}
+
+bool TabBoxHandlerImpl::checkMinimized(TabBoxClient* client) const
+{
+    switch (config().clientMinimizedMode()) {
+    case TabBoxConfig::ExcludeMinimizedClients:
+        return !client->isMinimized();
+    case TabBoxConfig::OnlyMinimizedClients:
+        return client->isMinimized();
+    default:       // TabBoxConfig::IgnoreMinimizedStatus
+        return true;
+    }
+}
+
+bool TabBoxHandlerImpl::checkMultiScreen(TabBoxClient* client) const
+{
+    Client* current = (static_cast< TabBoxClientImpl* >(client))->client();
     Workspace* workspace = Workspace::self();
+
+    switch (config().clientMultiScreenMode()) {
+    case TabBoxConfig::IgnoreMultiScreen:
+        return true;
+    case TabBoxConfig::ExcludeCurrentScreenClients:
+        return current->screen() != workspace->activeScreen();
+    default:       // TabBoxConfig::OnlyCurrentScreenClients
+        return current->screen() == workspace->activeScreen();
+    }
+}
+
+QWeakPointer<TabBoxClient> TabBoxHandlerImpl::clientToAddToList(TabBoxClient* client, int desktop) const
+{
     Client* ret = NULL;
     Client* current = (static_cast< TabBoxClientImpl* >(client))->client();
-    bool addClient = false;
-    bool applications = (config().clientListMode() == TabBoxConfig::AllDesktopsApplicationList ||
-                         config().clientListMode() == TabBoxConfig::CurrentDesktopApplicationList);
-    if (allDesktops)
-        addClient = true;
-    else
-        addClient = current->isOnDesktop(desktop);
-    addClient = addClient && current->isOnCurrentActivity();
+
+    bool addClient = checkDesktop(client, desktop)
+                  && checkActivity(client)
+                  && checkApplications(client)
+                  && checkMinimized(client)
+                  && checkMultiScreen(client);
     addClient = addClient && current->wantsTabFocus() && !current->skipSwitcher();
     if (addClient) {
         // don't add windows that have modal dialogs
@@ -144,25 +235,6 @@ QWeakPointer<TabBoxClient> TabBoxHandlerImpl::clientToAddToList(TabBoxClient* cl
         else {
             // nothing
         }
-        if (ret && applications) {
-            // check if the list already contains an entry of this application
-            foreach (QWeakPointer<TabBoxClient> tabBoxClientPointer, clientList()) {
-                QSharedPointer<TabBoxClient> tabBoxClient = tabBoxClientPointer.toStrongRef();
-                if (!tabBoxClient) {
-                    continue;
-                }
-                if (TabBoxClientImpl* c = dynamic_cast< TabBoxClientImpl* >(tabBoxClient.data())) {
-                    if (c->client()->resourceClass() == ret->resourceClass()) {
-                        ret = NULL;
-                        break;
-                    }
-                }
-            }
-        }
-    }
-    if (options->separateScreenFocus && options->xineramaEnabled) {
-        if (current->screen() != workspace->activeScreen())
-            ret = NULL;
     }
     if (ret)
         return ret->tabBoxClient();
@@ -172,10 +244,12 @@ QWeakPointer<TabBoxClient> TabBoxHandlerImpl::clientToAddToList(TabBoxClient* cl
 
 TabBoxClientList TabBoxHandlerImpl::stackingOrder() const
 {
-    ClientList stacking = Workspace::self()->stackingOrder();
+    ToplevelList stacking = Workspace::self()->stackingOrder();
     TabBoxClientList ret;
-    foreach (const Client * client, stacking) {
-        ret.append(client->tabBoxClient());
+    foreach (Toplevel *toplevel, stacking) {
+        if (Client *client = qobject_cast<Client*>(toplevel)) {
+            ret.append(client->tabBoxClient());
+        }
     }
     return ret;
 }
@@ -191,11 +265,23 @@ void TabBoxHandlerImpl::restack(TabBoxClient *c, TabBoxClient *under)
                                static_cast<TabBoxClientImpl*>(under)->client());
 }
 
+void TabBoxHandlerImpl::elevateClient(TabBoxClient *c, WId tabbox, bool b) const
+{
+    if (effects) {
+        const Client *cl = static_cast<TabBoxClientImpl*>(c)->client();
+        if (EffectWindow *w = static_cast<EffectsHandlerImpl*>(effects)->findWindow(cl->window()))
+            static_cast<EffectsHandlerImpl*>(effects)->setElevatedWindow(w, b);
+        if (EffectWindow *w = static_cast<EffectsHandlerImpl*>(effects)->findWindow(tabbox))
+            static_cast<EffectsHandlerImpl*>(effects)->setElevatedWindow(w, b);
+    }
+}
+
 
 QWeakPointer<TabBoxClient> TabBoxHandlerImpl::desktopClient() const
 {
-    foreach (const Client * client, Workspace::self()->stackingOrder()) {
-        if (client->isDesktop() && client->isOnCurrentDesktop() && client->screen() == Workspace::self()->activeScreen()) {
+    foreach (Toplevel *toplevel, Workspace::self()->stackingOrder()) {
+        Client *client = qobject_cast<Client*>(toplevel);
+        if (client && client->isDesktop() && client->isOnCurrentDesktop() && client->screen() == Workspace::self()->activeScreen()) {
             return client->tabBoxClient();
         }
     }
@@ -217,13 +303,18 @@ QVector< Window > TabBoxHandlerImpl::outlineWindowIds() const
     return Workspace::self()->outline()->windowIds();
 }
 
+void TabBoxHandlerImpl::activateAndClose()
+{
+    m_tabBox->accept();
+}
 
 /*********************************************************
 * TabBoxClientImpl
 *********************************************************/
 
-TabBoxClientImpl::TabBoxClientImpl()
+TabBoxClientImpl::TabBoxClientImpl(Client *client)
     : TabBoxClient()
+    , m_client(client)
 {
 }
 
@@ -241,6 +332,9 @@ QString TabBoxClientImpl::caption() const
 
 QPixmap TabBoxClientImpl::icon(const QSize& size) const
 {
+    if (m_client->isDesktop()) {
+        return KIcon("user-desktop").pixmap(size);
+    }
     return m_client->icon(size);
 }
 
@@ -274,6 +368,20 @@ int TabBoxClientImpl::height() const
     return m_client->height();
 }
 
+bool TabBoxClientImpl::isCloseable() const
+{
+    return m_client->isCloseable();
+}
+
+void TabBoxClientImpl::close()
+{
+    m_client->closeWindow();
+}
+
+bool TabBoxClientImpl::isFirstInTabBox() const
+{
+    return m_client->isFirstInTabBox();
+}
 
 /*********************************************************
 * TabBox
@@ -283,45 +391,65 @@ TabBox::TabBox(QObject *parent)
     , m_displayRefcount(0)
     , m_desktopGrab(false)
     , m_tabGrab(false)
+    , m_noModifierGrab(false)
     , m_forcedGlobalMouseGrab(false)
     , m_ready(false)
 {
     m_isShown = false;
     m_defaultConfig = TabBoxConfig();
     m_defaultConfig.setTabBoxMode(TabBoxConfig::ClientTabBox);
-    m_defaultConfig.setClientListMode(TabBoxConfig::CurrentDesktopClientList);
+    m_defaultConfig.setClientDesktopMode(TabBoxConfig::OnlyCurrentDesktopClients);
+    m_defaultConfig.setClientActivitiesMode(TabBoxConfig::OnlyCurrentActivityClients);
+    m_defaultConfig.setClientApplicationsMode(TabBoxConfig::AllWindowsAllApplications);
+    m_defaultConfig.setClientMinimizedMode(TabBoxConfig::IgnoreMinimizedStatus);
+    m_defaultConfig.setShowDesktopMode(TabBoxConfig::DoNotShowDesktopClient);
+    m_defaultConfig.setClientMultiScreenMode(TabBoxConfig::IgnoreMultiScreen);
     m_defaultConfig.setClientSwitchingMode(TabBoxConfig::FocusChainSwitching);
     m_defaultConfig.setLayout(TabBoxConfig::VerticalLayout);
 
     m_alternativeConfig = TabBoxConfig();
     m_alternativeConfig.setTabBoxMode(TabBoxConfig::ClientTabBox);
-    m_alternativeConfig.setClientListMode(TabBoxConfig::AllDesktopsClientList);
+    m_alternativeConfig.setClientDesktopMode(TabBoxConfig::AllDesktopsClients);
+    m_alternativeConfig.setClientActivitiesMode(TabBoxConfig::OnlyCurrentActivityClients);
+    m_alternativeConfig.setClientApplicationsMode(TabBoxConfig::AllWindowsAllApplications);
+    m_alternativeConfig.setClientMinimizedMode(TabBoxConfig::IgnoreMinimizedStatus);
+    m_alternativeConfig.setShowDesktopMode(TabBoxConfig::DoNotShowDesktopClient);
+    m_alternativeConfig.setClientMultiScreenMode(TabBoxConfig::IgnoreMultiScreen);
     m_alternativeConfig.setClientSwitchingMode(TabBoxConfig::FocusChainSwitching);
     m_alternativeConfig.setLayout(TabBoxConfig::VerticalLayout);
 
+    m_defaultCurrentApplicationConfig = m_defaultConfig;
+    m_defaultCurrentApplicationConfig.setClientApplicationsMode(TabBoxConfig::AllWindowsCurrentApplication);
+
+    m_alternativeCurrentApplicationConfig = m_alternativeConfig;
+    m_alternativeCurrentApplicationConfig.setClientApplicationsMode(TabBoxConfig::AllWindowsCurrentApplication);
+
     m_desktopConfig = TabBoxConfig();
     m_desktopConfig.setTabBoxMode(TabBoxConfig::DesktopTabBox);
     m_desktopConfig.setShowTabBox(true);
-    m_desktopConfig.setShowDesktop(false);
+    m_desktopConfig.setShowDesktopMode(TabBoxConfig::DoNotShowDesktopClient);
     m_desktopConfig.setDesktopSwitchingMode(TabBoxConfig::MostRecentlyUsedDesktopSwitching);
     m_desktopConfig.setLayout(TabBoxConfig::VerticalLayout);
 
     m_desktopListConfig = TabBoxConfig();
     m_desktopListConfig.setTabBoxMode(TabBoxConfig::DesktopTabBox);
     m_desktopListConfig.setShowTabBox(true);
-    m_desktopListConfig.setShowDesktop(false);
+    m_desktopListConfig.setShowDesktopMode(TabBoxConfig::DoNotShowDesktopClient);
     m_desktopListConfig.setDesktopSwitchingMode(TabBoxConfig::StaticDesktopSwitching);
     m_desktopListConfig.setLayout(TabBoxConfig::VerticalLayout);
     m_tabBox = new TabBoxHandlerImpl(this);
-    connect(m_tabBox, SIGNAL(ready()), SLOT(handlerReady()));
+    QTimer::singleShot(0, this, SLOT(handlerReady()));
+    connect(m_tabBox, SIGNAL(selectedIndexChanged()), SIGNAL(itemSelected()));
 
     m_tabBoxMode = TabBoxDesktopMode; // init variables
     connect(&m_delayedShowTimer, SIGNAL(timeout()), this, SLOT(show()));
     connect(Workspace::self(), SIGNAL(configChanged()), this, SLOT(reconfigure()));
+    QDBusConnection::sessionBus().registerObject("/TabBox", this, QDBusConnection::ExportScriptableContents);
 }
 
 TabBox::~TabBox()
 {
+    QDBusConnection::sessionBus().unregisterObject("/TabBox");
 }
 
 void TabBox::handlerReady()
@@ -351,6 +479,10 @@ void TabBox::initShortcuts(KActionCollection* keys)
     KEY(I18N_NOOP("Walk Through Windows (Reverse)"),       Qt::ALT + Qt::SHIFT + Qt::Key_Backtab, slotWalkBackThroughWindows(), m_cutWalkThroughWindowsReverse, slotWalkBackThroughWindowsKeyChanged(QKeySequence))
     KEY(I18N_NOOP("Walk Through Windows Alternative"),     0, slotWalkThroughWindowsAlternative(), m_cutWalkThroughWindowsAlternative, slotWalkThroughWindowsAlternativeKeyChanged(QKeySequence))
     KEY(I18N_NOOP("Walk Through Windows Alternative (Reverse)"), 0, slotWalkBackThroughWindowsAlternative(), m_cutWalkThroughWindowsAlternativeReverse, slotWalkBackThroughWindowsAlternativeKeyChanged(QKeySequence))
+    KEY(I18N_NOOP("Walk Through Windows of Current Application"), Qt::ALT + Qt::Key_QuoteLeft, slotWalkThroughCurrentAppWindows(), m_cutWalkThroughCurrentAppWindows, slotWalkThroughCurrentAppWindowsKeyChanged(QKeySequence))
+    KEY(I18N_NOOP("Walk Through Windows of Current Application (Reverse)"), Qt::ALT + Qt::Key_AsciiTilde, slotWalkBackThroughCurrentAppWindows(), m_cutWalkThroughCurrentAppWindowsReverse, slotWalkBackThroughCurrentAppWindowsKeyChanged(QKeySequence))
+    KEY(I18N_NOOP("Walk Through Windows of Current Application Alternative"), 0, slotWalkThroughCurrentAppWindowsAlternative(), m_cutWalkThroughCurrentAppWindowsAlternative, slotWalkThroughCurrentAppWindowsAlternativeKeyChanged(QKeySequence))
+    KEY(I18N_NOOP("Walk Through Windows of Current Application Alternative (Reverse)"), 0, slotWalkBackThroughCurrentAppWindowsAlternative(), m_cutWalkThroughCurrentAppWindowsAlternativeReverse, slotWalkBackThroughCurrentAppWindowsAlternativeKeyChanged(QKeySequence))
     KEY(I18N_NOOP("Walk Through Desktops"),                0, slotWalkThroughDesktops(), m_cutWalkThroughDesktops, slotWalkThroughDesktopsKeyChanged(QKeySequence))
     KEY(I18N_NOOP("Walk Through Desktops (Reverse)"),      0, slotWalkBackThroughDesktops(), m_cutWalkThroughDesktopsReverse, slotWalkBackThroughDesktopsKeyChanged(QKeySequence))
     KEY(I18N_NOOP("Walk Through Desktop List"),            0, slotWalkThroughDesktopList(), m_cutWalkThroughDesktopList, slotWalkThroughDesktopListKeyChanged(QKeySequence))
@@ -373,6 +505,12 @@ void TabBox::setMode(TabBoxMode mode)
     case TabBoxWindowsAlternativeMode:
         m_tabBox->setConfig(m_alternativeConfig);
         break;
+    case TabBoxCurrentAppWindowsMode:
+        m_tabBox->setConfig(m_defaultCurrentApplicationConfig);
+        break;
+    case TabBoxCurrentAppWindowsAlternativeMode:
+        m_tabBox->setConfig(m_alternativeCurrentApplicationConfig);
+        break;
     case TabBoxDesktopMode:
         m_tabBox->setConfig(m_desktopConfig);
         break;
@@ -550,6 +688,11 @@ void TabBox::reconfigure()
     loadConfig(c->group("TabBox"), m_defaultConfig);
     loadConfig(c->group("TabBoxAlternative"), m_alternativeConfig);
 
+    m_defaultCurrentApplicationConfig = m_defaultConfig;
+    m_defaultCurrentApplicationConfig.setClientApplicationsMode(TabBoxConfig::AllWindowsCurrentApplication);
+    m_alternativeCurrentApplicationConfig = m_alternativeConfig;
+    m_alternativeCurrentApplicationConfig.setClientApplicationsMode(TabBoxConfig::AllWindowsCurrentApplication);
+
     m_tabBox->setConfig(m_defaultConfig);
 
     m_delayShow = config.readEntry<bool>("ShowDelay", true);
@@ -558,8 +701,18 @@ void TabBox::reconfigure()
 
 void TabBox::loadConfig(const KConfigGroup& config, TabBoxConfig& tabBoxConfig)
 {
-    tabBoxConfig.setClientListMode(TabBoxConfig::ClientListMode(
-                                       config.readEntry<int>("ListMode", TabBoxConfig::defaultListMode())));
+    tabBoxConfig.setClientDesktopMode(TabBoxConfig::ClientDesktopMode(
+                                       config.readEntry<int>("DesktopMode", TabBoxConfig::defaultDesktopMode())));
+    tabBoxConfig.setClientActivitiesMode(TabBoxConfig::ClientActivitiesMode(
+                                       config.readEntry<int>("ActivitiesMode", TabBoxConfig::defaultActivitiesMode())));
+    tabBoxConfig.setClientApplicationsMode(TabBoxConfig::ClientApplicationsMode(
+                                       config.readEntry<int>("ApplicationsMode", TabBoxConfig::defaultApplicationsMode())));
+    tabBoxConfig.setClientMinimizedMode(TabBoxConfig::ClientMinimizedMode(
+                                       config.readEntry<int>("MinimizedMode", TabBoxConfig::defaultMinimizedMode())));
+    tabBoxConfig.setShowDesktopMode(TabBoxConfig::ShowDesktopMode(
+                                       config.readEntry<int>("ShowDesktopMode", TabBoxConfig::defaultShowDesktopMode())));
+    tabBoxConfig.setClientMultiScreenMode(TabBoxConfig::ClientMultiScreenMode(
+                                       config.readEntry<int>("MultiScreenMode", TabBoxConfig::defaultMultiScreenMode())));
     tabBoxConfig.setClientSwitchingMode(TabBoxConfig::ClientSwitchingMode(
                                             config.readEntry<int>("SwitchingMode", TabBoxConfig::defaultSwitchingMode())));
 
@@ -569,8 +722,6 @@ void TabBox::loadConfig(const KConfigGroup& config, TabBoxConfig& tabBoxConfig)
                                TabBoxConfig::defaultShowTabBox()));
     tabBoxConfig.setHighlightWindows(config.readEntry<bool>("HighlightWindows",
                                      TabBoxConfig::defaultHighlightWindow()));
-    tabBoxConfig.setShowDesktop(config.readEntry<bool>("ShowDesktop",
-                                TabBoxConfig::defaultShowDesktop()));
 
     tabBoxConfig.setLayoutName(config.readEntry<QString>("LayoutName", TabBoxConfig::defaultLayoutName()));
 }
@@ -627,36 +778,16 @@ bool TabBox::handleMouseEvent(XEvent* e)
             close();  // click outside closes tab
             return true;
         }
-    }
-    if (m_tabBoxMode == TabBoxWindowsMode || m_tabBoxMode == TabBoxWindowsAlternativeMode) {
-        // pass to declarative view
-        return false;
-    }
-
-    // not declarative view
-    if (e->type != ButtonPress) {
-        return true;
-    }
-    QPoint pos(e->xbutton.x_root, e->xbutton.y_root);
-    QModelIndex index;
-    if (e->xbutton.button == Button1 || e->xbutton.button == Button2 || e->xbutton.button == Button3) {
-        index = m_tabBox->indexAt(pos);
-        if (e->xbutton.button == Button2 && index.isValid()) {
-            if (TabBoxClientImpl* client = static_cast< TabBoxClientImpl* >(m_tabBox->client(index))) {
-                if (Workspace::self()->hasClient(client->client())) {
-                    client->client()->closeWindow();
-                    return true;
-                }
+        if (e->xbutton.button == Button5 || e->xbutton.button == Button4) {
+            // mouse wheel event
+            const QModelIndex index = m_tabBox->nextPrev(e->xbutton.button == Button5);
+            if (index.isValid()) {
+                setCurrentIndex(index);
             }
+            return true;
         }
-    } else {
-        // mouse wheel event
-        index = m_tabBox->nextPrev(e->xbutton.button == Button5);
     }
-
-    if (index.isValid())
-        setCurrentIndex(index);
-    return true;
+    return false;
 }
 
 void TabBox::grabbedKeyEvent(QKeyEvent* event)
@@ -666,7 +797,13 @@ void TabBox::grabbedKeyEvent(QKeyEvent* event)
         // tabbox has been replaced, check effects
         return;
     }
-    setCurrentIndex(m_tabBox->grabbedKeyEvent(event));
+    if (m_noModifierGrab) {
+        if (event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return || event->key() == Qt::Key_Space) {
+            accept();
+            return;
+        }
+    }
+    m_tabBox->grabbedKeyEvent(event);
 }
 
 /*!
@@ -800,6 +937,38 @@ void TabBox::slotWalkBackThroughWindowsAlternative()
     navigatingThroughWindows(false, m_cutWalkThroughWindowsAlternativeReverse, TabBoxWindowsAlternativeMode);
 }
 
+void TabBox::slotWalkThroughCurrentAppWindows()
+{
+    if (!m_ready){
+        return;
+    }
+    navigatingThroughWindows(true, m_cutWalkThroughCurrentAppWindows, TabBoxCurrentAppWindowsMode);
+}
+
+void TabBox::slotWalkBackThroughCurrentAppWindows()
+{
+    if (!m_ready){
+        return;
+    }
+    navigatingThroughWindows(false, m_cutWalkThroughCurrentAppWindowsReverse, TabBoxCurrentAppWindowsMode);
+}
+
+void TabBox::slotWalkThroughCurrentAppWindowsAlternative()
+{
+    if (!m_ready){
+        return;
+    }
+    navigatingThroughWindows(true, m_cutWalkThroughCurrentAppWindowsAlternative, TabBoxCurrentAppWindowsAlternativeMode);
+}
+
+void TabBox::slotWalkBackThroughCurrentAppWindowsAlternative()
+{
+    if (!m_ready){
+        return;
+    }
+    navigatingThroughWindows(false, m_cutWalkThroughCurrentAppWindowsAlternativeReverse, TabBoxCurrentAppWindowsAlternativeMode);
+}
+
 void TabBox::slotWalkThroughDesktops()
 {
     if (!m_ready){
@@ -909,6 +1078,26 @@ void TabBox::slotWalkBackThroughWindowsAlternativeKeyChanged(const QKeySequence&
     m_cutWalkThroughWindowsAlternativeReverse = KShortcut(seq);
 }
 
+void TabBox::slotWalkThroughCurrentAppWindowsKeyChanged(const QKeySequence& seq)
+{
+    m_cutWalkThroughCurrentAppWindows = KShortcut(seq);
+}
+
+void TabBox::slotWalkBackThroughCurrentAppWindowsKeyChanged(const QKeySequence& seq)
+{
+    m_cutWalkThroughCurrentAppWindowsReverse = KShortcut(seq);
+}
+
+void TabBox::slotWalkThroughCurrentAppWindowsAlternativeKeyChanged(const QKeySequence& seq)
+{
+    m_cutWalkThroughCurrentAppWindowsAlternative = KShortcut(seq);
+}
+
+void TabBox::slotWalkBackThroughCurrentAppWindowsAlternativeKeyChanged(const QKeySequence& seq)
+{
+    m_cutWalkThroughCurrentAppWindowsAlternativeReverse = KShortcut(seq);
+}
+
 void TabBox::modalActionsSwitch(bool enabled)
 {
     QList<KActionCollection*> collections;
@@ -920,11 +1109,61 @@ void TabBox::modalActionsSwitch(bool enabled)
     action->setEnabled(enabled);
 }
 
+void TabBox::open(bool modal, const QString &layout)
+{
+    if (isDisplayed()) {
+        return;
+    }
+    if (modal) {
+        if (!establishTabBoxGrab()) {
+            return;
+        }
+        m_tabGrab = true;
+    } else {
+        m_tabGrab = false;
+    }
+    m_noModifierGrab = !modal;
+    setMode(TabBoxWindowsMode);
+    if (!layout.isNull()) {
+        TabBoxConfig tempConfig;
+        tempConfig = tabBox->config();
+        tempConfig.setLayoutName(layout);
+        tempConfig.setShowTabBox(true);
+        tabBox->setConfig(tempConfig);
+    }
+    reset();
+    show();
+}
+
+void TabBox::openEmbedded(qulonglong wid, QPoint offset, QSize size, int horizontalAlignment, int verticalAlignment, const QString &layout)
+{
+    if (isDisplayed()) {
+        return;
+    }
+    m_tabGrab = false;
+    m_noModifierGrab = true;
+    tabBox->setEmbedded(static_cast<WId>(wid));
+    tabBox->setEmbeddedOffset(offset);
+    tabBox->setEmbeddedSize(size);
+    tabBox->setEmbeddedAlignment(static_cast<Qt::AlignmentFlag>(horizontalAlignment) | static_cast<Qt::AlignmentFlag>(verticalAlignment));
+    setMode(TabBoxWindowsMode);
+    if (!layout.isNull()) {
+        TabBoxConfig tempConfig;
+        tempConfig = tabBox->config();
+        tempConfig.setLayoutName(layout);
+        tabBox->setConfig(tempConfig);
+    }
+    reset();
+    show();
+}
+
 bool TabBox::startKDEWalkThroughWindows(TabBoxMode mode)
 {
     if (!establishTabBoxGrab())
         return false;
     m_tabGrab = true;
+    m_noModifierGrab = false;
+    tabBox->resetEmbedded();
     modalActionsSwitch(false);
     setMode(mode);
     reset();
@@ -936,6 +1175,7 @@ bool TabBox::startWalkThroughDesktops(TabBoxMode mode)
     if (!establishTabBoxGrab())
         return false;
     m_desktopGrab = true;
+    m_noModifierGrab = false;
     modalActionsSwitch(false);
     setMode(mode);
     reset();
@@ -974,8 +1214,8 @@ void TabBox::CDEWalkThroughWindows(bool forward)
     for (int i = Workspace::self()->stackingOrder().size() - 1;
             i >= 0 ;
             --i) {
-        Client* it = Workspace::self()->stackingOrder().at(i);
-        if (it->isOnCurrentActivity() && it->isOnCurrentDesktop() && !it->isSpecialWindow()
+        Client* it = qobject_cast<Client*>(Workspace::self()->stackingOrder().at(i));
+        if (it && it->isOnCurrentActivity() && it->isOnCurrentDesktop() && !it->isSpecialWindow()
                 && it->isShown(false) && it->wantsTabFocus()
                 && !it->keepAbove() && !it->keepBelow()) {
             c = it;
@@ -1009,7 +1249,7 @@ void TabBox::CDEWalkThroughWindows(bool forward)
             Workspace::self()->lowerClient(c);
         if (options->focusPolicyIsReasonable()) {
             Workspace::self()->activateClient(nc);
-            if (nc->isShade() && options->shadeHover)
+            if (nc->isShade() && options->isShadeHover())
                 nc->setShade(ShadeActivated);
         } else {
             if (!nc->isOnDesktop(currentDesktop()))
@@ -1026,7 +1266,7 @@ void TabBox::KDEOneStepThroughWindows(bool forward, TabBoxMode mode)
     nextPrev(forward);
     if (Client* c = currentClient()) {
         Workspace::self()->activateClient(c);
-        if (c->isShade() && options->shadeHover)
+        if (c->isShade() && options->isShadeHover())
             c->setShade(ShadeActivated);
     }
 }
@@ -1061,15 +1301,36 @@ void TabBox::keyPress(int keyQt)
     if (m_tabGrab) {
         KShortcut forwardShortcut;
         KShortcut backwardShortcut;
-        if (mode() == TabBoxWindowsMode) {
-            forwardShortcut = m_cutWalkThroughWindows;
-            backwardShortcut = m_cutWalkThroughWindowsReverse;
-        } else {
-            forwardShortcut = m_cutWalkThroughWindowsAlternative;
-            backwardShortcut = m_cutWalkThroughWindowsAlternativeReverse;
+        switch (mode()) {
+            case TabBoxWindowsMode:
+                forwardShortcut = m_cutWalkThroughWindows;
+                backwardShortcut = m_cutWalkThroughWindowsReverse;
+                break;
+            case TabBoxWindowsAlternativeMode:
+                forwardShortcut = m_cutWalkThroughWindowsAlternative;
+                backwardShortcut = m_cutWalkThroughWindowsAlternativeReverse;
+                break;
+            case TabBoxCurrentAppWindowsMode:
+                forwardShortcut = m_cutWalkThroughCurrentAppWindows;
+                backwardShortcut = m_cutWalkThroughCurrentAppWindowsReverse;
+                break;
+            case TabBoxCurrentAppWindowsAlternativeMode:
+                forwardShortcut = m_cutWalkThroughCurrentAppWindowsAlternative;
+                backwardShortcut = m_cutWalkThroughCurrentAppWindowsAlternativeReverse;
+                break;
+            default:
+                kDebug(125) << "Invalid TabBoxMode";
+                return;
         }
         forward = forwardShortcut.contains(keyQt);
         backward = backwardShortcut.contains(keyQt);
+        if (!forward && !backward) {
+            // if the shortcuts do not match, try matching again after filtering the shift key from keyQt
+            // it is needed to handle correctly the ALT+~ shorcut for example as it is coded as ALT+SHIFT+~ in keyQt
+            keyQt &= ~Qt::ShiftModifier;
+            forward = forwardShortcut.contains(keyQt);
+            backward = backwardShortcut.contains(keyQt);
+        }
         if (forward || backward) {
             kDebug(125) << "== " << forwardShortcut.toString()
                         << " or " << backwardShortcut.toString() << endl;
@@ -1098,11 +1359,32 @@ void TabBox::keyPress(int keyQt)
 
 void TabBox::close(bool abort)
 {
-    removeTabBoxGrab();
+    if (isGrabbed()) {
+        removeTabBoxGrab();
+    }
     hide(abort);
     modalActionsSwitch(true);
     m_tabGrab = false;
     m_desktopGrab = false;
+    m_noModifierGrab = false;
+}
+
+void TabBox::accept()
+{
+    Client* c = currentClient();
+    close();
+    if (c) {
+        Workspace::self()->activateClient(c);
+        if (c->isShade() && options->isShadeHover())
+            c->setShade(ShadeActivated);
+        if (c->isDesktop())
+            Workspace::self()->setShowingDesktop(!Workspace::self()->showingDesktop());
+    }
+}
+
+void TabBox::reject()
+{
+    close(true);
 }
 
 /*!
@@ -1110,6 +1392,9 @@ void TabBox::close(bool abort)
  */
 void TabBox::keyRelease(const XKeyEvent& ev)
 {
+    if (m_noModifierGrab) {
+        return;
+    }
     unsigned int mk = ev.state &
                       (KKeyServer::modXShift() |
                        KKeyServer::modXCtrl() |
@@ -1143,16 +1428,8 @@ void TabBox::keyRelease(const XKeyEvent& ev)
         return;
     if (m_tabGrab) {
         bool old_control_grab = m_desktopGrab;
-        Client* c = currentClient();
-        close();
+        accept();
         m_desktopGrab = old_control_grab;
-        if (c) {
-            Workspace::self()->activateClient(c);
-            if (c->isShade() && options->shadeHover)
-                c->setShade(ShadeActivated);
-            if (c->isDesktop())
-                Workspace::self()->setShowingDesktop(!Workspace::self()->showingDesktop());
-        }
     }
     if (m_desktopGrab) {
         bool old_tab_grab = m_tabGrab;
