@@ -258,7 +258,8 @@ QRectF DesktopToolBox::boundingRect() const
 
     //keep space for the label and a character more
     if (!m_containment->activity().isNull()) {
-        extraSpace = Plasma::Theme::defaultTheme()->fontMetrics().width(m_containment->activity()+'x');
+        extraSpace = iconSize().width()
+            + Plasma::Theme::defaultTheme()->fontMetrics().width(m_containment->activity()+'x');
     }
 
     qreal left, top, right, bottom;
@@ -364,7 +365,7 @@ void DesktopToolBox::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
                               m_background->contentsRect().width()));
 
         textRect = QStyle::alignedRect(QApplication::layoutDirection(), Qt::AlignRight|Qt::AlignVCenter, textSize, boundRect);
-        textRect.moveTopLeft(textRect.topLeft() + QPoint(rect.top(), rect.left()));
+        textRect.moveTopLeft(textRect.topLeft() + QPoint(rect.top() - iconSize().height(), rect.left()));
     } else {
         Qt::Alignment alignment;
 
@@ -377,7 +378,7 @@ void DesktopToolBox::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
         iconRect = QStyle::alignedRect(QApplication::layoutDirection(), alignment, iconSize(), m_background->contentsRect().toRect());
 
         textRect = QStyle::alignedRect(QApplication::layoutDirection(), Qt::AlignRight|Qt::AlignVCenter, textSize, m_background->contentsRect().toRect());
-        textRect.moveTopLeft(textRect.topLeft() + rect.topLeft().toPoint());
+        textRect.moveTopLeft(textRect.topLeft() + QPoint(rect.top() - iconSize().height(), rect.left()));
     }
 
     iconRect.moveTopLeft(iconRect.topLeft() + rect.topLeft().toPoint());
